import sys, getopt
import argparse

import numpy as np
import numpy.ma as ma

import xarray as xr

import cartopy.crs as ccrs
import cartopy.feature as cfeature
import cartopy.util as cutil
from cartopy.mpl.gridliner import LONGITUDE_FORMATTER, LATITUDE_FORMATTER

import matplotlib.pyplot as plt
import matplotlib.patches as mpatches
import matplotlib.ticker as mticker

import cmocean
import dask

def one_map_noproj(fig, nx, ny, pos, data, unit, mask, cmap, vmin, vmax, title):

    datam=np.ma.array(data,mask=1-mask)

    ax = fig.add_subplot(nx,ny,pos)

    pcolor=ax.pcolormesh(datam,cmap=cmap,vmin=vmin,vmax=vmax)
    fig.subplots_adjust(right=0.8)

    cbar = plt.colorbar(pcolor,orientation='horizontal',shrink=0.75, pad=0.1,label=unit)
    ax.set_title(title,size=17,y=1.08)

def get_data2D(filei,var,ttime,lev):
    ds=xr.open_dataset(filei)
    if (lev == -1):
        data=ds[var][ttime].squeeze()
    else:
        data=ds[var][ttime,lev].squeeze()
    return data

def sections_latitude(ax,seclat,seclon1,seclon2,datam,navlon,navlev,cmap,deplim,vmin,vmax,unit,title):
    lon=navlon[seclat,seclon1:seclon2]
    if (np.mean(np.diff(lon))<0):
        lon[np.where(lon<0)]=lon[np.where(lon<0)]+360
    pcolor=ax.pcolormesh(lon,navlev,datam[:,seclat,seclon1:seclon2],vmin=vmin,vmax=vmax,cmap=cmap, shading='nearest')
    ax.set_ylim([0,deplim])
    ax.invert_yaxis()
    plt.xlabel('longitude °E')
    cbar = plt.colorbar(pcolor,orientation='horizontal',shrink=0.75, pad=0.1,label=unit)
    ax.set_title(title,size=17,y=1.08)

def sections_longitude(ax,seclon,seclat1,seclat2,datam,navlat,navlev,cmap,deplim,vmin,vmax,unit,title):
    lat=navlat[seclat1:seclat2,seclon]
    pcolor=ax.pcolormesh(lat,navlev,datam[:,seclat1:seclat2,seclon],vmin=vmin,vmax=vmax,cmap=cmap, shading='nearest')
    ax.set_ylim([0,deplim])
    ax.invert_yaxis()
    plt.xlabel('latitude °N')
    cbar = plt.colorbar(pcolor,orientation='horizontal',shrink=0.75, pad=0.1,label=unit)
    ax.set_title(title,size=17,y=1.08)

def sections_latitude_nolims(ax,seclat,seclon1,seclon2,datam,navlon,navlev,cmap,deplim,unit,title):
    lon=navlon[seclat,seclon1:seclon2]
    if (np.mean(np.diff(lon))<0):
        lon[np.where(lon<0)]=lon[np.where(lon<0)]+360
    pcolor=ax.pcolormesh(lon,navlev,datam[:,seclat,seclon1:seclon2],cmap=cmap, shading='nearest')
    ax.set_ylim([0,deplim])
    ax.invert_yaxis()
    plt.xlabel('longitude °E')
    cbar = plt.colorbar(pcolor,orientation='horizontal',shrink=0.75, pad=0.1,label=unit)
    ax.set_title(title,size=17,y=1.08)

def sections_longitude_nolims(ax,seclon,seclat1,seclat2,datam,navlat,navlev,cmap,deplim,unit,title):
    lat=navlat[seclat1:seclat2,seclon]
    pcolor=ax.pcolormesh(lat,navlev,datam[:,seclat1:seclat2,seclon],cmap=cmap, shading='nearest')
    ax.set_ylim([0,deplim])
    ax.invert_yaxis()
    plt.xlabel('latitude °N')
    cbar = plt.colorbar(pcolor,orientation='horizontal',shrink=0.75, pad=0.1,label=unit)
    ax.set_title(title,size=17,y=1.08)

def all_sections(fig,nbcol,ncol,nblin,nlin,datam,navlon,navlat,navlev,cmap,unit,varname,vmin,vmax):
    odd=ncol%nbcol
    ax = fig.add_subplot(nblin,nbcol,nlin-odd)
    sections_latitude(ax,9181,4062,4135,datam,navlon,navlev,cmap,70,vmin[0],vmax[0],unit,str(varname)+' Bering section y='+str(np.int64(np.round(navlat[9181,4062].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+nbcol-odd)
    sections_latitude(ax,10145,9600,9945,datam,navlon,navlev,cmap,5000,vmin[1],vmax[1],unit,str(varname)+' Fram section y='+str(np.int64(np.round(navlat[10145,9600].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+2*nbcol-odd)
    sections_latitude(ax,9955,8331,8812,datam,navlon,navlev,cmap,2500,vmin[2],vmax[2],unit,str(varname)+' Baffin section y='+str(np.int64(np.round(navlat[9955,8331].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+3*nbcol-odd)
    sections_latitude(ax,7154,7446,7498,datam,navlon,navlev,cmap,900,vmin[3],vmax[3],unit,str(varname)+' Florida-Bahamas section y='+str(np.int64(np.round(navlat[7154,7446].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+4*nbcol-odd)
    sections_latitude(ax,5563,11763,11934,datam,navlon,navlev,cmap,3500,vmin[4],vmax[4],unit,str(varname)+' Mozambique section y='+str(np.int64(np.round(navlat[5563,11763].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+5*nbcol-odd)
    sections_latitude(ax,5258,11406,12959,datam,navlon,navlev,cmap,6000,vmin[5],vmax[5],unit,str(varname)+' Africa-Australia1 section y='+str(np.int64(np.round(navlat[5258,11406].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+6*nbcol-odd)
    sections_latitude(ax,5258,0,1578,datam,navlon,navlev,cmap,6000,vmin[6],vmax[6],unit,str(varname)+' Africa-Australia2 section y='+str(np.int64(np.round(navlat[5258,0].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+7*nbcol-odd)
    sections_latitude(ax,5258,2835,7797,datam,navlon,navlev,cmap,6000,vmin[7],vmax[7],unit,str(varname)+' Australia-America section y='+str(np.int64(np.round(navlat[5258,2835].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+8*nbcol-odd)
    sections_latitude(ax,5258,8634,10861,datam,navlon,navlev,cmap,6000,vmin[8],vmax[8],unit,str(varname)+' America-Africa section y='+str(np.int64(np.round(navlat[5258,8634].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+9*nbcol-odd)
    sections_longitude(ax,7435,7003,7106,datam,navlat,navlev,cmap,1200,vmin[9],vmax[9],unit,str(varname)+' Cuba-Florida section x='+str(np.int64(np.round(navlon[7003,7435].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+10*nbcol-odd)
    sections_longitude(ax,2725,2732,4696,datam,navlat,navlev,cmap,5000,vmin[10],vmax[10],unit,str(varname)+' Australia-Antartica section x='+str(np.int64(np.round(navlon[2732,2725].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+11*nbcol-odd)
    sections_longitude(ax,1524,5359,5854,datam,navlat,navlev,cmap,6000,vmin[11],vmax[11],unit,str(varname)+' Indonesian throughflow section x='+str(np.int64(np.round(navlon[5359,1524].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+12*nbcol-odd)
    sections_longitude(ax,11050,2576,4832,datam,navlat,navlev,cmap,6000,vmin[12],vmax[12],unit,str(varname)+' South Africa section x='+str(np.int64(np.round(navlon[2576,11050].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+13*nbcol-odd)
    sections_longitude(ax,12823,2794,4108,datam,navlat,navlev,cmap,5000,vmin[13],vmax[13],unit,str(varname)+' Kerguelen section x='+str(np.int64(np.round(navlon[2794,12823].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+14*nbcol-odd)
    sections_longitude(ax,3504,2386,4300,datam,navlat,navlev,cmap,6000,vmin[14],vmax[14],unit,str(varname)+' Campbell section x='+str(np.int64(np.round(navlon[2386,3504].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+15*nbcol-odd)
    sections_longitude(ax,7914,2954,3790,datam,navlat,navlev,cmap,6000,vmin[15],vmax[15],unit,str(varname)+' Drake section x='+str(np.int64(np.round(navlon[2954,7914].values)))+'°E')

def all_sections_nolims(fig,nbcol,ncol,nblin,nlin,datam,navlon,navlat,navlev,cmap,unit,varname):
    odd=ncol%nbcol
    ax = fig.add_subplot(nblin,nbcol,nlin-odd)
    sections_latitude_nolims(ax,9181,4062,4135,datam,navlon,navlev,cmap,70,unit,str(varname)+' Bering section y='+str(np.int64(np.round(navlat[9181,4062].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+nbcol-odd)
    sections_latitude_nolims(ax,10145,9600,9945,datam,navlon,navlev,cmap,5000,unit,str(varname)+' Fram section y='+str(np.int64(np.round(navlat[10145,9600].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+2*nbcol-odd)
    sections_latitude_nolims(ax,9955,8331,8812,datam,navlon,navlev,cmap,2500,unit,str(varname)+' Baffin section y='+str(np.int64(np.round(navlat[9955,8331].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+3*nbcol-odd)
    sections_latitude_nolims(ax,7154,7446,7498,datam,navlon,navlev,cmap,900,unit,str(varname)+' Florida-Bahamas section y='+str(np.int64(np.round(navlat[7154,7446].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+4*nbcol-odd)
    sections_latitude_nolims(ax,5563,11763,11934,datam,navlon,navlev,cmap,3500,unit,str(varname)+' Mozambique section y='+str(np.int64(np.round(navlat[5563,11763].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+5*nbcol-odd)
    sections_latitude_nolims(ax,5258,11406,12959,datam,navlon,navlev,cmap,6000,unit,str(varname)+' Africa-Australia1 section y='+str(np.int64(np.round(navlat[5258,11406].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+6*nbcol-odd)
    sections_latitude_nolims(ax,5258,0,1578,datam,navlon,navlev,cmap,6000,unit,str(varname)+' Africa-Australia2 section y='+str(np.int64(np.round(navlat[5258,0].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+7*nbcol-odd)
    sections_latitude_nolims(ax,5258,2835,7797,datam,navlon,navlev,cmap,6000,unit,str(varname)+' Australia-America section y='+str(np.int64(np.round(navlat[5258,2835].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+8*nbcol-odd)
    sections_latitude_nolims(ax,5258,8634,10861,datam,navlon,navlev,cmap,6000,unit,str(varname)+' America-Africa section y='+str(np.int64(np.round(navlat[5258,8634].values)))+'°N')
    ax = fig.add_subplot(nblin,nbcol,nlin+9*nbcol-odd)
    sections_longitude_nolims(ax,7435,7003,7106,datam,navlat,navlev,cmap,1200,unit,str(varname)+' Cuba-Florida section x='+str(np.int64(np.round(navlon[7003,7435].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+10*nbcol-odd)
    sections_longitude_nolims(ax,2725,2732,4696,datam,navlat,navlev,cmap,5000,unit,str(varname)+' Australia-Antartica section x='+str(np.int64(np.round(navlon[2732,2725].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+11*nbcol-odd)
    sections_longitude_nolims(ax,1524,5359,5854,datam,navlat,navlev,cmap,6000,unit,str(varname)+' Indonesian throughflow section x='+str(np.int64(np.round(navlon[5359,1524].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+12*nbcol-odd)
    sections_longitude_nolims(ax,11050,2576,4832,datam,navlat,navlev,cmap,6000,unit,str(varname)+' South Africa section x='+str(np.int64(np.round(navlon[2576,11050].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+13*nbcol-odd)
    sections_longitude_nolims(ax,12823,2794,4108,datam,navlat,navlev,cmap,5000,unit,str(varname)+' Kerguelen section x='+str(np.int64(np.round(navlon[2794,12823].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+14*nbcol-odd)
    sections_longitude_nolims(ax,3504,2386,4300,datam,navlat,navlev,cmap,6000,unit,str(varname)+' Campbell section x='+str(np.int64(np.round(navlon[2386,3504].values)))+'°E')
    ax = fig.add_subplot(nblin,nbcol,nlin+15*nbcol-odd)
    sections_longitude_nolims(ax,7914,2954,3790,datam,navlat,navlev,cmap,6000,unit,str(varname)+' Drake section x='+str(np.int64(np.round(navlon[2954,7914].values)))+'°E')

